# clear workspace
rm( list=ls() ); graphics.off()
setwd("C:\\Dokumente und Einstellungen\\Enrico Schumann\\Eigene Dateien\\Projects\\myth")

# define Nelson--Siegel--Svensson model
NSS <- function(betaV,mats) {
    # betaV = beta1-4, lambda1-2
    gam1 <- mats / betaV[5]
    gam2 <- mats / betaV[6]
    aux1 <- 1 - exp(-gam1)
    aux2 <- 1 - exp(-gam2)
    y <- betaV[1] + betaV[2] * (aux1 / gam1) + 
            betaV[3] * (aux1 / gam1 + aux1 - 1) + 
            betaV[4] * (aux2 / gam2 + aux2 - 1) 
    return(y)
}

# define objective function
OF <- function(betaV,dataList) {		
    mats  <- dataList$mats
    yM    <- dataList$yM
    model <- dataList$model
    y     <- model(betaV,mats)
    aux   <- y - yM
    aux   <- crossprod(aux)
    return(aux)
}

# get bliss/diebold/li data
x <- url("http://www.ssc.upenn.edu/~fdiebold/papers/paper49/FBFITTED.txt")
open(x); dili <- scan(x, skip = 14); close(x)
mat <- NULL
for (i in 1:372) {mat <- rbind(mat,dili[(19*(i-1)+1):(19*(i-1)+19)])}
mats  <- c(1,3,6,9,12,15,18,21,24,30,36,48,60,72,84,96,108,120)/12

# the obligatory perspective plot
persp(x = mat[,1], y = mats,mat[,-1],
        phi = 40, theta = 20, ticktype = "detailed",
        xlab = "time", ylab = "time to maturity in years",
        zlab = "zero rates in %")

# settings: minimum, maximum, number of parameters
settings <- list(
        min = c( 0,-15,-30,-30,0,3),
        max = c(15, 30, 30, 30,3,6),
        d = 6)

# how many restarts per month?
trials <- 100

# set array to store results
res <- array(NA, c(372,trials));goodRes <- array(NA, c(372,1))

# run through all months
set.seed(75325428)
howFar <- txtProgressBar(min=1,max=372,style=3)
for(t in 1:372){
    # market yields
    yM <- as.numeric(mat[t,-1])
    dataList <- list(yM = yM, mats = mats, model = NSS)
    # random starting values
    for( rr in seq(trials) ) {
        s0 <- settings$min + 
                (settings$max - settings$min) * runif(settings$d)
        sol <- nlminb(s0, OF, 
                data = dataList,
                lower = settings$min, upper = settings$max)
        res[t,rr] <- sqrt(sum((NSS(sol$par,mats)-yM)^2)/18)
    }
    # good starting values
    s0 <- c((yM[18]+yM[17])/2,yM[1]-(yM[18]+yM[17])/2,0,0,1,1)
    sol <- nlminb(s0,OF, 
            data = dataList, 
            lower = settings$min, upper = settings$max, 
            control = list(eval.max=50000,iter.max=50000) )
    goodRes[t] <- sqrt(sum((NSS(sol$par,mats)-yM)^2)/18)
    #
    setTxtProgressBar(howFar, value=t)
}
close(howFar)
# compute difference in basis points between random-start solutions and good-start solutions
diffs <- 100 * (apply(res,1,min) - goodRes)
labs <-  rep(NA,372); labs[seq(from=1, to= 372, by = 60)] <- seq(1970,2000,5)
diffs <- diffs[372:1]; labs <- labs[372:1] # flip upside down for plotting

# plot
#setEPS()
#postscript("errors.eps",family="Palatino",width=4, height=5)
par(bty="n",las=1,ps=10,mar=c(4,4,1,1))
plot(diffs,1:372,type="n",xlab="error difference in basis points",yaxt="n",ylab="")
points(diffs,1:372,cex=.25,pch=19)
axis(2,at=1:372,labels=labs,lty=0)
#dev.off()